/**************************************************************
    LCD I2C for PIC16F687(PIC16F689)
    Strawberry linux I2C LCD SB1602B
    for ACM1602N1 16 columns 2 lines Hardware
 **************************************************************
    author air_variable
    http://www.ne.jp/asahi/air/variable/
    Tools:
    MPLABX v3.00
    MPLAB XC8 complier version 1.34
---------------------------------------------------------------
    This program is new header for Hitech PICC.
    When you use an old compiler, you will have to correct some descriptions.
---------------------------------------------------------------
    Date        Time    Version Note
    20120715    0900    1.00    Begin the program
    20120717    0840    1.10    Fix port initialize
    20120917    1920    1.20    initialize CGRAM area
    20120928    0911    1.30    includes LCD negative voltage generate OSC
    20130908    2320    1.50    Add process "when case of SSPSTATbit.R_nW == 1"
    20150523    2320    1.60    Fixed wrong brackets position on line-75 
---------------------------------------------------------------
    ATTENTION!
    The PIC microcontroller on ACM1602N1 is already protected. therefore cannot read this one.
    The original firmware cannot return, if you owerwrite it.
    When you want to return in the future, please write in after exchanging chips before.
    No operations are guaranteed although this firmware is programming to become equivalent.

 **************************************************************/

//#include <htc.h>
#include <xc.h>
#include <stdio.h>
#include "main.h"
#include "lcd.h"
#include "promdata.h"
#include "eeprom.h"

//CONFIG
/*  This is an old setting of configration. See main.h .
__CONFIG(
        FOSC_INTRCIO & WDTE_OFF & PWRTE_ON & MCLRE_OFF & CP_OFF & CPD_OFF & BOREN_ON & IESO_OFF & FCMEN_OFF
        );
 */

//Global settings
//Fifo Data
unsigned char Buffer_FIFO[MAX_BUFF];
unsigned char FIFO_write_pointer = 0;
unsigned char FIFO_read_pointer = 0;
unsigned char FIFO_datasize = 0;
//unsigned char i2c_2ndflag =0; // first is RS , second is data

void interrupt handler(void) //interrupt handle
{
    unsigned char temp;
    unsigned char data;
    //Handle I2C recieve interrupt

    if (SSPIF == 1) { // Occurring I2C interrupt
        if (SSPSTATbits.R_nW == 0) {
            if (SSPSTATbits.D_nA == 0) {
                // receive address field
                temp = SSPBUF; // dummy read data
            } else {
                // receive data field
                data = SSPBUF; // Read data (ACK is respond automatically)
                if (FIFO_datasize < MAX_BUFF) { // check FIFO buffer capacitance
                    Buffer_FIFO[FIFO_write_pointer] = data; //set a receice data
                    FIFO_write_pointer = (FIFO_write_pointer + 1) & (MAX_BUFF - 1); //ring buff
                    FIFO_datasize++;
                }
            }
            SSPIF = 0; // Interrupt Flag clear
            SSPCONbits.CKP = 1; // Release SCL and prepare the next receive.
        } else { // SSPSTATbit.R_nW == 1 ***1.50 ***1.60
            temp = SSPBUF;
            SSPBUF = temp; //dummy Write
            SSPIF = 0;
            SSPCONbits.CKP = 1;
        }
    }

    //	if (BCLIF == 1) { //PIC16F68x are not support
    //		BCLIF = 0 ;   // if I2C bus collision occurred.(None of action)
    //	}

    //timer interval
    if (T0IF == 1) { //timer interval interrupt
        if (LCD_Negative_Volt_OSC == 1) LCD_Negative_Volt_OSC = 0;
        else LCD_Negative_Volt_OSC = 1; //LCD Negative Voltage generator
        TMR0 = Interval_timer0_value;
        T0IF = 0;
    }
    //I2C unknown error reset
    if (SSPCONbits.WCOL == 1) SSPCONbits.WCOL = 0; // I2C collision reset
    if (SSPCONbits.SSPOV == 1) SSPCONbits.SSPOV = 0; // I2C overflow reset

}

unsigned char Buffer_readdata(void) {
    unsigned char wdat;
    //read FIFO buffer
    while (FIFO_datasize == 0); // pause if buffer is empty
    wdat = Buffer_FIFO[FIFO_read_pointer];
    FIFO_read_pointer = (FIFO_read_pointer + 1)&(MAX_BUFF - 1); //ring buff
    FIFO_datasize--;
    return wdat; //get wdat
    //	return FIFO_datasize;//debug
}

/*
void Buffer_check(void){
        unsigned char data;
        if (FIFO_datasize==MAX_BUFF-1) {
                FIFO_datasize 		=0;	//FIFO clear
                FIFO_write_pointer 	=0;
                FIFO_read_pointer 	=0;
                if (SSPCONbits.WCOL ==1) SSPCONbits.WCOL = 0;	// I2C collision reset
                delay_10ms(20);//200ms
        }
}
 */
void delay_10ms(unsigned char temp) { //10ms multiple x times
    while (temp--) {
        __delay_ms(10);
    }
}

void InitI2C_Slave(int address) {
    SSPSTAT = 0b00000000; // I2C speed is 100kHz as normal speed ***1.60
    SSPCON = 0b00110110; // I2C slave mode use SDA/SCL pin adress7bit mode
    //	SSPCON2bits.SEN  = 1;		// SCL control (PIC16F68x are not support)
    SSPADD = address << 1; // address setting (Left shift)
    //	SSPMSK = 0b11111110 ;		// address compare mask
    //	SSPIE  = 1 ;				// enable SSP(I2C) interrupt
    //	BCLIE  = 1 ;				// enable MSSP(I2C) collision interrupt
    //	PEIE   = 1 ;				// enable peripheral interrupt
    //	GIE    = 1 ;				// enable all the interrupt
    SSPIF = 0; // clear SSP(I2C) flag
    //	BCLIF  = 0 ;				// clear MSSP(I2C) collision flag(PIC16F68x are not support)
}

//Write one character (correspond printf)

void putch(unsigned char c) {
    lcd_putch(c);
    return;
}

void lcd_CG_init(void) { //**1.20
    unsigned char i, j;
    lcd_write(0x40); //CG RAM mode
    for (i = 0; i < 9; i++) { //8CGRAMs clear
        for (j = 0; j < 8; j++) {
            lcd_cgwrite(0x00); //clear CG RAM 8stacks
        }
    }
    lcd_write(0x80); //set DD RAM
    lcd_goto(0x00); //set LCD line1
    return;
}


//------------------main

main() {
    // Register
    unsigned char temp, data;

    //	Port initalize
    //		  76543210
    PORTA = 0b00001011; //Port Initalize
    PORTB = 0b11110000; //Port Initalize
    PORTC = 0b00000000; //Port Initalize
    TRISA = 0b11001011; //
    TRISB = 0b01111111; //	***1.10 RB7 is output
    TRISC = 0b00000000; //LCD output ,see lcd.h

    ANSEL = 0; //All port are use for digital port
    ANSELH = 0; //All port are use for digital port
    nRABPU = 0; //PORTA/B weak pullups are enable
    WPUA = 0b00000011; //Pullup RA0,RA1 enable ***1.10
    WPUB = 0b10100000; //Pullup RB5,RB7 enable ***1.10
    OSCCON = 0b01110000; //PLL off,8MHz,IntOSC

    //Init
    //      1/((8MHz/4 )/2)=1usec  set 64 count (6.089KHz)
    OPTION_REG = 0b00000000; // prescale 1/2 RABPU= EN
    GIE = 0; //Int off

    //timer0 setting (interrupt event timer 3.328ms)
    T0IE = 0; //timer interrupt is off
    TMR0 = Interval_timer0_value; //interrupt timer counter set

    //LCD init
    delay_10ms(2); //stabilize time **1.20
    lcd_init(); // **1.20
    lcd_clear(); // **1.20
    lcd_CG_init(); // **1.20
    //        lcd_goto(0);
    //        lcd_puts("test message");
    //        printf("Test message");
    //        while(1); //loop forever

    //I2C init
    if (RB5 == 1) temp = E2promRead(eeprom_I2C_ID0); // compatible ACM1602N1 address default
    else temp = E2promRead(eeprom_I2C_ID1); // 2nd ID
    InitI2C_Slave(temp); // compatible ACM1602N1 address
    printf("[I2C %02X] SB1602B", temp);
    //interrupt enable
    T0IE = 1; //timer interrupt on
    SSPIE = 1; // enable SSP(I2C) interrupt
    //	BCLIE = 1 ;				// enable MSSP(I2C) collision interrupt (PIC16F68x are not support)
    PEIE = 1; //Peripheral interrupt Enable
    GIE = 1; // enable all the interrupt

    while (1) {
        data = Buffer_readdata(); //Buffer read 1'st
        //		printf("%02x",data);
        if ((data & 0x40) == 0x40) // RS check
            LCD_RS = 1; // RS =1
        else LCD_RS = 0; // RS =0

        data = Buffer_readdata(); //Buffer read 2'nd
        //		printf("%02x",data);
        lcd_primitive_write(data); //write LCD data



    }
}
