/*----------------------------------------------------------------------------
	LCD I2C for PIC16F689(PIC16F687)
	Strawberry linux I2C LCD SB1602B
	for ACM1602N1 16 columns 2 lines Hardware
----------------------------------------------------------------------------*/

/*******************************************/
/*              define data                */
/*******************************************/
/*
---------------------------------------------------------------
	Pin assignment
	 1:VDD
	 2:RA5:O:LCD_RW
	 3:RA4:O:LCD_E
	 4:RA3:I:(MCLR) switch with Pullup *see define PORT
	 5:RC5:O:LCD_D5
	 6:RC4:O:LCD_D4
	 7:RC3:O:LCD_D3
	 8:RC6:O:LCD_D6
	 9:RC7:O:LCD_D7
	10:RB7:O:(TX) LCD negative voltage generator
	20:VSS
	19:RA0:O:(ICSPD)
	18:RA1:O:(ICSPC)
	17:RA2:O:LCD_RS
	16:RC0:O:LCD_D0
	15:RC1:O:LCD_D1
	14:RC2:O:LCD_D2
	13:RB4:O:(SDA) switch *see define PORT
	12:RB5:O:(RX)
	11:RB6:O:(SCK) switch *see define PORT
*/


// PIC16F689 Configuration Bit Settings

#include <xc.h>

// CONFIG
#pragma config FOSC = INTRCIO   // Oscillator Selection bits (INTOSCIO oscillator: I/O function on RA4/OSC2/CLKOUT pin, I/O function on RA5/OSC1/CLKIN)
#pragma config WDTE = OFF       // Watchdog Timer Enable bit (WDT disabled and can be enabled by SWDTEN bit of the WDTCON register)
#pragma config PWRTE = ON       // Power-up Timer Enable bit (PWRT enabled)
#pragma config MCLRE = OFF      // MCLR Pin Function Select bit (MCLR pin function is digital input, MCLR internally tied to VDD)
#pragma config CP = OFF         // Code Protection bit (Program memory code protection is disabled)
#pragma config CPD = OFF        // Data Code Protection bit (Data memory code protection is disabled)
#pragma config BOREN = ON       // Brown-out Reset Selection bits (BOR enabled)
#pragma config IESO = OFF       // Internal External Switchover bit (Internal External Switchover mode is disabled)
#pragma config FCMEN = OFF      // Fail-Safe Clock Monitor Enabled bit (Fail-Safe Clock Monitor is disabled)



//LCD kind set------------------------------------------------------------------- 
//Choose use the LCD module model number

//#define	_LCD0802
#define	_LCD1602
//#define	_LCD2402 

//------------------------------------------------------------------- 
//Clock control
#define MHz 000000
#define _XTAL_FREQ 8MHz

//Version infomation(for use display)
#define	VersionCode	"16F687/689 V1.60"
#ifdef	_LCD0802
#define	VersionInfo	"ACM0802C"
#endif
#ifdef	_LCD1602
#define	VersionInfo	"ACM1602 Hardware"
#endif
#ifdef	_LCD2402
#define	VersionInfo	"SC242A(16F1827) "
#endif

//RX buffer size
#define MAX_BUFF	32

//Device address setting
//#define I2C_deviceID 0x3e //Strawberry linux I2C LCD SB1602B


//LCD negative voltage generator
#define LCD_Negative_Volt_OSC   RB7
#define Interval_timer0_value   192

/*******************************************/
/*              table data                 */
/*******************************************/


/*******************************************/
/*           extern function               */
/*******************************************/


/*******************************************/
/*            extern variable              */
/*******************************************/
extern void delay_10ms(unsigned char);
